clearvars;
close all;
clc;

% General Settings
resistor_series_name    = 'E24';
capacitor_series_name   = 'E6';    

f_osc   = 500e3; % Oscillator frequency
period  = 1 / f_osc;
V_ref   = 5.1; % Reference voltage
Vin     = 230 * sqrt(2); % Input voltage (RMS to peak conversion)
Id      = 7.5; % Drive current
Is      = 20; % Peak current
Rs      = 0.05; % Sense resistor value
V_out   = 80; % Output voltage
R1_fb   = 3000; % Feedback resistor 1
Ct      = 5e-9;
delta_I = 0.1; % Current ripple
delta_V = 0.01; % Voltage ripple

syms Rt td R2 R2_fb

% Capacitance and Rt calculation
eq1 = f_osc / 1e3 == 2.2 / ((Ct / 1e-6) * (Rt / 1e3));
sol = solve(eq1, Rt);
Rt = double(sol);

% td calculation
eq2 = td == 145 * (Ct / 1e-6) * (Id / (Id - 3.6 / (Rt / 1e3))); 
sol = solve(eq2, td);
td = double(sol); % Microseconds 

% Rounding considerations
Rt_new = round63(Rt, resistor_series_name);
td_new = 145 * (Ct / 1e-6) * (Id / (Id - 3.6 / (Rt_new / 1e3)));

% Auto-restart settings
auto_restart_bool = false;

if auto_restart_bool
    I_set = 0.4e-3;
else
    I_set = 5e-3;
end 

R1 = V_ref / I_set;

% Rounding considerations
R1 = round63(R1, resistor_series_name);
I_set_new = V_ref / R1;

% R2 calculation for peak current
eq1 = Is == ((R2 * V_ref) / (R1 + R2) - 0.5) / (3 * Rs);
sol = solve(eq1, R2);
R2 = double(sol);

% Rounding considerations
R2_new = round63(R2, resistor_series_name);
Is_new = ((R2_new * V_ref) / (R1 + R2_new) - 0.5) / (3 * Rs);

% Feedback calculations
eq1 = V_out == V_ref * (1 + R1_fb / R2_fb);
sol = solve(eq1, R2_fb);
R2_fb = double(sol);
P_fb = V_out / (R1_fb + R2_fb);

% Rounding considerations
R2_fb_new = round63(R2_fb, resistor_series_name);

% Inductor and capacitor calculations
D = V_out / Vin; % Duty cycle
L = (Vin - V_out) / (f_osc * delta_I); % Inductance
C = delta_I / (8 * f_osc * delta_V); % Capacitance

% Rounding considerations
L_new = round63(L, resistor_series_name);
C_new = round63(C, capacitor_series_name);
V_out_new = V_ref * (1 + R1_fb / R2_fb_new);
P_fb_new = V_out_new / (R1_fb + R2_fb_new);

% Display Results
fprintf('=== Calculated Results ===\n');
fprintf('Oscillator Frequency: %.2f kHz\n', f_osc / 1e3);
fprintf('Ct (Chosen): %.2f nF\n', Ct / 1e-9);
fprintf('Rt (Initial): %.2f Ohms\n', Rt);
fprintf('Rt (Rounded): %.2f Ohms\n', Rt_new);
fprintf('td (Initial): %.2f µs\n', td);
fprintf('td (Rounded): %.2f µs\n\n', td_new);

fprintf('Auto Restart: %s\n', string(auto_restart_bool));
fprintf('I_set (Chosen): %.2f mA\n', I_set * 1e3);
fprintf('I_set (Rounded): %.2f mA\n\n', I_set_new * 1e3);

fprintf('RS (Chosen): %.3f Ohms\n', Rs);
fprintf('R1 (Chosen): %.2f Ohms\n', R1);
fprintf('R2 (Initial): %.2f Ohms\n', R2);
fprintf('R2 (Rounded): %.2f Ohms\n', R2_new);
fprintf('Is (New): %.2f A\n', Is_new);
fprintf('Is (Chosen): %.2f A\n\n', Is);

fprintf('Feedback Voltage Output (Initial): %.2f V\n', V_out);
fprintf('R1_fb (Chosen): %.2f Ohms\n', R1_fb);
fprintf('R2_fb (Initial): %.2f Ohms\n', R2_fb);
fprintf('R2_fb (Rounded): %.2f Ohms\n', R2_fb_new);
fprintf('Feedback Voltage Output (New): %.2f V\n', V_out_new);
fprintf('Feedback Power (New): %.2f W\n\n', P_fb_new);

fprintf('Duty Cycle: %.2f\n', D);
fprintf('Inductance (Initial): %.2f µH\n', L * 1e6);
fprintf('Inductance (Rounded): %.2f µH\n', L_new * 1e6);
fprintf('Capacitance (Initial): %.2f µF\n', C * 1e6);
fprintf('Capacitance (Rounded): %.2f µF\n', C_new * 1e6);
